defmodule Mix.Tasks.Migrator.Import.Notifications do
  use Mix.Task
  import Mix.Pleroma
  import Mix.Migrator
  alias Pleroma.Repo
  alias Pleroma.User
  alias Pleroma.Activity
  alias Pleroma.Notification

  @shortdoc "Import notifications."
  def run(_) do
    start_pleroma()

    File.stream!("migrator/notifications.txt")
    |> Enum.each(&handle_line/1)
  end

  defp handle_line(line) do
    params =
      Jason.decode!(line)
      |> keys_to_atoms
      |> loop_fields([:inserted_at, :updated_at], &parse_timestamp/1)
      |> maybe_fix_user_id
      |> maybe_fix_activity_id

    try_create_notification(params)
  end

  defp maybe_fix_user_id(params) do
    user = User.get_by_ap_id(params[:user_ap_id])

    if user do
      Map.put(params, :user_id, user.id)
      |> Map.delete(:user_ap_id)
    else
      params
    end
  end

  defp maybe_fix_activity_id(params) do
    activity = Activity.get_by_ap_id(params[:activity_ap_id])

    if activity do
      Map.put(params, :activity_id, activity.id)
      |> Map.delete(:activity_ap_id)
    else
      params
    end
  end

  defp try_create_notification(%{user_id: _, activity_id: _} = params) do
    changeset = struct(Notification, params)

    try do
      {:ok, _notification} = Repo.insert(changeset)
      shell_info("Notification created")
    rescue
      Ecto.ConstraintError -> shell_info("Notification already exists, skipping")
      MatchError -> shell_info("Could not create notification")
      FunctionClauseError -> shell_info("Could not create notification")
    end
  end

  defp try_create_notification(_) do
    shell_info("Notification could not be imported - skipped")
  end
end
